import {{application_name|title}}{{model_name|title}}ServiceBase from './{{model_name}}.service.base';

export default class {{application_name|title}}{{model_name|title}}Service extends {{application_name|title}}{{model_name|title}}ServiceBase {

}
